#' Extract Genus Information from AlgaeBase
#'
#' This function extracts genus-level information from AlgaeBase, either from a
#' genus name or a URL. Optionally, it retrieves species-level data.
#'
#' @param genus Character vector of genus names.
#' @param URL Character vector of URLs pointing to genus pages.
#' @param getspecies Logical; if TRUE, retrieves species-level information.
#' @param nested Logical; if TRUE, nests the results for each genus.
#' @param gebaseurl The base URL for AlgaeBase genus search.
#'
#' @return A tibble containing the genus and species information.
#' @importFrom XML xpathSApply htmlParse
#' @importFrom dplyr tibble bind_rows mutate row_number pull full_join filter
#'   select group_by everything
#' @importFrom tidyr nest
#' @importFrom httr GET
#' @importFrom readr parse_number
#' @export
get_genusab <- function(genus = NULL,
                        URL = NULL,
                        getspecies = FALSE,
                        nested = TRUE,
                        gebaseurl = "https://www.algaebase.org/search/?genus=") {
  if (length(genus) > 1) {
    retdfs <- dplyr::tibble()
    for (igenus in seq_along(genus)) {
      print(paste0(igenus, "/", length(genus), ": ", genus[igenus], "..."))
      retdfs <- dplyr::bind_rows(
        retdfs,
        get_genusab(
          genus = genus[igenus],
          getspecies = getspecies,
          nested = FALSE,
          gebaseurl = gebaseurl
        )
      )
    }
    if (nested) {
      retdfs <- retdfs |>
        dplyr::mutate(ROW_ID = dplyr::row_number()) |>
        tidyr::nest(taxonomy = -ROW_ID) |>
        dplyr::pull(taxonomy)
    }
    return(retdfs)
  }
  if (length(URL) > 1) {
    retdfs <- dplyr::tibble()
    for (iURL in seq_along(URL)) {
      print(paste0(iURL, "/", length(URL), ": ", URL[iURL], "..."))
      retdfs <- dplyr::bind_rows(
        retdfs,
        get_genusab(
          URL = URL[iURL],
          getspecies = getspecies,
          nested = FALSE,
          gebaseurl = gebaseurl
        )
      )
    }
    if (nested) {
      retdfs <- retdfs |>
        dplyr::mutate(ROW_ID = dplyr::row_number()) |>
        tidyr::nest(taxonomy = -ROW_ID) |>
        dplyr::pull(taxonomy)
    }
    return(retdfs)
  }
  if (is.null(URL)) {
    if (is.null(genus)) {
      warning("At least a valid genus name or a valid URL with genus information should be provided!")
      return(NULL)
    }
    URL <- paste0(gebaseurl, genus)
  }

  parsed <- XML::htmlParse(httr::GET(URL))
  rooturl <- get_rooturl(gebaseurl)
  resmodestr <- XML::xpathSApply(parsed, "//div[@id = 'resultsnav']", XML::xmlValue) |>
    trimws(which = "both")

  if (length(resmodestr) == 0) {
    message("Found match!")

    if (is.null(genus)) {
      genus <- XML::xpathSApply(parsed, "//h1/em", XML::xmlValue)
    }

    if (!getspecies) {
      retdf <- get_taxlevel(parsed) |>
        dplyr::mutate(genus = genus)
      return(retdf)
    }
    typesp <- XML::xpathSApply(
      parsed, "//div[contains(@class, '-class')]/strong/a",
      XML::xmlValue
    )
    retdf <- dplyr::tibble(
      level = XML::xpathSApply(parsed, "//div[contains(@class, '-class')]/@class") |>
        as.character(),
      link = XML::xpathSApply(parsed, "//div[contains(@class, '-class')]//a/@href") |>
        as.character(),
      taxname = XML::xpathSApply(
        parsed, "//div[contains(@class, '-class')]//a",
        XML::xmlValue
      ),
      status = XML::xpathSApply(
        parsed, "//div[contains(@class, '-class')]//b",
        XML::xmlValue
      )
    ) |>
      dplyr::mutate(typesp = identical(taxname, typesp)) |>
      dplyr::mutate(abspid = readr::parse_number(gsub(
        pattern = "^.*species_id=(\\d*).*$",
        replacement = "\\1",
        x = link
      ))) |>
      dplyr::mutate(level = gsub(pattern = "-class", replacement = "", x = level)) |>
      dplyr::mutate(spname = "")

    for (irow in seq_len(nrow(retdf))) {
      retdf$spname[irow] <- ifelse(retdf$level[irow] == "infraspecific",
        retdf$spname[irow - 1], retdf$taxname[irow]
      )
    }

    retdf <- retdf |>
      dplyr::filter(level == "infraspecific") |>
      dplyr::select(spname, taxname, abspid, status, link) |>
      dplyr::group_by(spname) |>
      tidyr::nest(infraspecific = c(taxname, abspid, link, status)) |>
      dplyr::full_join(
        retdf |> dplyr::filter(level == "species") |>
          dplyr::select(taxname, abspid, status, typesp, link) |>
          dplyr::rename(spname = taxname),
        by = "spname"
      ) |>
      dplyr::mutate(genus = genus) |>
      tidyr::nest(species = -genus) |>
      dplyr::bind_cols(... = get_taxlevel(parsed), ... = _)
    if (nested) {
      retdf <- retdf |>
        tidyr::nest(taxonomy = dplyr::everything()) |>
        dplyr::pull(taxonomy)
    }

    return(retdf)
  }

  if (resmodestr == "") {
    warning("No matches found!")
    return(NULL)
  }

  if (any(grepl("Found - Displaying", resmodestr))) {
    message("More than 1 genus match found!")

    if (is.null(genus)) {
      warning("Argument `genus` not given!")
      return(NULL)
    }

    URL <- dplyr::tibble(
      genus = XML::xpathSApply(parsed, "//tr//a", XML::xmlValue),
      link = XML::xpathSApply(parsed, "//tr//a/@href") |> as.character()
    ) |>
      dplyr::filter(grepl(pattern = paste0("^", genus, "$"), genus)) |>
      dplyr::pull(.data$link) |>
      paste0(... = rooturl, ... = "search/genus/", ... = _)

    genusab <- get_genusab(
      URL = URL,
      getspecies = getspecies,
      nested = nested,
      gebaseurl = gebaseurl
    )
    return(genusab)
  }
}
