#' Get Taxonomic Data
#'
#' Retrieves taxonomic information from an online database based on a species
#' name.
#'
#' @param taxname A character vector containing the taxonomic name to search.
#' @param gettaxlevel Logical, if TRUE, fetches taxonomic classification levels.
#' @param getabid Logical, if TRUE, also returns species id (abid).
#' @param headless Logical, if TRUE, runs in headless mode for browser-based
#'        interactions.
#' @param spbaseurl Base URL for the species search (default is Algaebase).
#' @return A tibble containing the taxonomic information and related data.
#' @importFrom httr GET
#' @importFrom XML htmlParse xpathSApply xmlValue
#' @importFrom dplyr tibble mutate bind_rows bind_cols select filter
#' @importFrom stringr str_c str_split
#' @importFrom tidyr pivot_wider
#' @importFrom readr parse_number
#' @importFrom purrr map2
#' @importFrom RSelenium rsDriver
#' @export
get_taxab <- function(taxname,
                      gettaxlevel = TRUE,
                      getabid = TRUE,
                      headless = FALSE,
                      spbaseurl = "https://www.algaebase.org/search/?species=") {
  if (length(taxname) > 1) {
    warning("Only support 1 tax!")
    return(NULL)
  }
  rooturl <- get_rooturl(spbaseurl)
  taxname <- gsub("flos-aquae", "flosaquae", taxname)
  tidytaxname <- get_tidytaxname(taxname, type = "tax")
  genus <- get_tidytaxname(taxname, type = "genus")
  species <- get_tidytaxname(taxname, type = "species")
  keystr <- gsub(" ", "%20", tidytaxname)
  # PATTERN <- paste0("^", tidytaxname)
  URL <- paste0(spbaseurl, keystr)
  parsed <- XML::htmlParse(httr::GET(URL))
  plain.text <- XML::xpathSApply(parsed, "//p", XML::xmlValue)
  status <- plain.text[grep("Status", plain.text)]
  # Additional status handling logic
  if (length(status) == 0) {
    if (species == "") {
      return(NULL)
    }
    sp.name.len <- nchar(species)
    sp.trunc <- ifelse(sp.name.len > 5,
      substr(species, 1, sp.name.len - 5),
      substr(species, 1, 2)
    )
    taxname.trunc <- paste(genus, sp.trunc)
    return(get_taxab(taxname.trunc,
      gettaxlevel = gettaxlevel,
      getabid = getabid,
      headless = headless,
      spbaseurl = spbaseurl
    ))
  }

  if (grepl("This name is of an entity that is currently accepted taxonomically", status)) {
    status <- 1
  }
  # Handle different taxonomic statuses and generate results
  if (status == 1) {
    synonym <- ifelse(getabid,
      list(get_synonymdf(parsed, getabid = TRUE)),
      get_synonymdf(parsed)
    )
    retdf <- dplyr::tibble(taxname = get_taxname(parsed)) |>
      dplyr::mutate(
        curtaxname = taxname,
        genus = genus,
        species = species,
        abid = get_abid(parsed),
        synonym = synonym,
        imginfo = list(get_taximginfo(parsed)),
        unchecked = FALSE,
        accepted = TRUE
      )

    if (!gettaxlevel) {
      return(retdf)
    }
    taxleveldf <- dplyr::bind_rows(get_taxlevel(parsed))
    retdf <- dplyr::bind_cols(retdf, taxleveldf)
    return(retdf)
  }
  # Additional logic for handling synonyms, unmatched names, etc.
  return(dplyr::tibble(
    genus = NA,
    species = NA,
    exact.match = 0,
    accepted = 0,
    synonyms = NA,
    orig.name = paste(genus, species),
    match.name = NA
  ))
}
