#' Retrieve Taxonomic Tree from Algaebase
#'
#' This function retrieves taxonomic information for a given taxonomic ID (taxid)
#' from Algaebase and constructs a taxonomic tree.
#'
#' @param taxid Numeric. The taxonomic ID for the desired entry on Algaebase.
#' @param level Integer. The depth level of the taxonomic tree (default is 1).
#' @param taxbaseurl Character. The base URL of the Algaebase taxonomy query
#' (default: "https://www.algaebase.org/browse/taxonomy/?id=").
#'
#' @importFrom dplyr tibble slice mutate_all rename_all bind_cols as_tibble
#' @importFrom stringr str_c str_split
#' @importFrom httr GET
#' @importFrom XML htmlParse xpathSApply xmlValue
#' @importFrom readr parse_number
#' @export
#' @return A tibble containing taxonomic information, with sub-trees in list form.
get_taxtree <- function(taxid,
                        level = 1,
                        taxbaseurl = "https://www.algaebase.org/browse/taxonomy/?id=") {
  IRUN <<- IRUN + 1 # Increment global run counter
  if (taxid %in% idlist) {
    message(paste0("Duplicated id: ", taxid, " - Level ", level))
    return(dplyr::tibble())
  }
  idlist <<- c(idlist, taxid)
  URL <- paste0(taxbaseurl, taxid)
  parsed <- XML::htmlParse(httr::GET(URL))
  nosub <- XML::xpathSApply(
    parsed,
    "//h3[contains(text(), 'Subordinate taxa')]",
    XML::xmlValue
  ) |>
    identical(list())
  if (nosub) {
    return(NULL)
  }
  hassub <- XML::xpathSApply(
    parsed,
    "//*[contains(text(), 'No subordinate taxa were found.')]",
    XML::xmlValue
  ) |>
    identical(list())
  if (!hassub) {
    return(dplyr::tibble())
  }
  Rank <- XML::xpathSApply(parsed, "//h3[i]/text()", XML::xmlValue) |>
    gsub(pattern = ": ", replacement = "_", x = _)
  retdf <- XML::xpathSApply(
    parsed,
    "//tr[td[h3[contains(text(), 'Subordinate taxa')]]]//following-sibling::tr[td[p]]//td[position() < 6]",
    XML::xmlValue
  ) |>
    trimws(which = "both") |>
    matrix(ncol = 5, byrow = TRUE) |>
    dplyr::as_tibble() |>
    dplyr::slice(-1) |>
    dplyr::mutate(dplyr::across(
      tidyselect::everything(),
      \(x) gsub(pattern = "\n.*$|\\s*$", replacement = "", x)
    )) |>
    dplyr::rename_all(~ c("Rank", "Name", "Authority", "Fossil", "nSpecies")) |>
    dplyr::bind_cols(
      XML::xpathSApply(
        parsed,
        "//tr[td[h3[contains(text(), 'Subordinate taxa')]]]//following-sibling::tr[td[p]]//td[position() = 2 or position() = 6]//a/@href"
      ) |>
        as.character() |>
        matrix(ncol = 2, byrow = TRUE) |>
        dplyr::as_tibble() |>
        dplyr::rename_all(~ c("taxLink", "taxonid"))
    ) |>
    dplyr::rename_with(\(x) paste0(Rank, x))
  retdf$subdf <- list(NULL)
  irun <- 1 # Initialize run counter
  for (irow in seq_len(nrow(retdf))) {
    if (!grepl("species_id", retdf[[6]][irow])) {
      message(paste0(
        "RUN", IRUN, " - Level",
        level, "-", irun, ": ", taxid, "-",
        readr::parse_number(retdf[[6]][irow])
      ))
      irun <- irun + 1
      retdf$subdf[[irow]] <- tryCatch(
        {
          get_taxtree(readr::parse_number(retdf[[6]][irow]), level = level + 1)
        },
        error = function(e) {
          dplyr::tibble(taxid = taxid, err = TRUE, info = e)
        }
      )
    }
  }
  if (level <= 2) {
    save(list = "retdf", file = paste0("level", level, "-", taxid, "tree.RData"))
    message(paste0("level", level, "-", taxid, "tree.RData saved!"))
  }
  return(retdf)
}
