#' Clean and Extract Taxonomic Names
#'
#' This function cleans up taxonomic names by removing unwanted characters and
#' substrings and formats them into genus and species. It can also return genus
#' only, or the full taxonomic name depending on the input parameters.
#'
#' @param taxname A character vector containing the taxonomic names to be
#' processed.
#' @param type A character string specifying the type of name to return. Default
#' is "tax". Other options include "genus" or "species".
#' @param addsp Logical; if TRUE, adds "sp." to genus-only names.
#'
#' @return A character vector with cleaned taxonomic names.
#' @importFrom dplyr tibble mutate pull
#' @export
#'
#' @examples
#' get_tidytaxname(c("Homo sapiens", "Panthera cf. leo", "Canis lupus var. familiaris"))
get_tidytaxname <- function(taxname,
                            type = "tax",
                            addsp = FALSE) {
  taxname <- taxname |>
    iconv(to = "ASCII//TRANSLIT") |>
    gsub(pattern = "A<", replacement = "e", x = _, fixed = TRUE) |>
    gsub(pattern = "Cfr. ", replacement = "", x = _, ignore.case = TRUE) |>
    gsub(pattern = "cf ", replacement = "", x = _, ignore.case = TRUE) |>
    gsub(pattern = "cf.", replacement = "", x = _, ignore.case = TRUE) |>
    gsub(pattern = "colony", replacement = "", x = _, ignore.case = TRUE) |>
    gsub(pattern = "colonies", replacement = "", x = _, ignore.case = TRUE) |>
    gsub(pattern = "cells", replacement = "", x = _, ignore.case = TRUE) |>
    gsub(pattern = "\\s{2,}", replacement = " ", x = _, ignore.case = TRUE) |>
    trimws(which = "both", x = _)
  genusonly_pat <- "\\ssp{1,2}[1-9\\.]{0,1}(\\s|$)"
  var_pat <- "\\s{1,}(var|subsp|ssp|v|morph|gr|mor|aff|f|tab)\\.{0,1}\\s"
  tidytaxname <- dplyr::tibble(taxname = taxname) |>
    dplyr::mutate(isgenus = grepl(genusonly_pat, taxname)) |>
    dplyr::mutate(isvar = grepl(var_pat, taxname)) |>
    dplyr::mutate(taxname2 = gsub(
      pattern = paste0(genusonly_pat, ".*$"),
      replacement = "", x = taxname
    )) |>
    dplyr::mutate(taxname2 = gsub(
      pattern = paste0("(", var_pat, "\\S*).*$"),
      replacement = "\\1", x = taxname2
    )) |>
    dplyr::mutate(taxname2 = ifelse(!isvar,
      gsub(
        pattern = "^(\\S{1,}\\s{1,}\\S{1,})\\s.*$",
        replacement = "\\1", x = taxname2
      ),
      taxname2
    )) |>
    dplyr::mutate(taxname2 = gsub(
      pattern = "^(\\S{1,}\\s{1,})[A-Z].*$",
      replacement = "\\1", x = taxname2
    )) |>
    dplyr::mutate(genus = gsub(
      pattern = "^(\\S{1,})\\s.*$",
      replacement = "\\1", x = taxname2
    )) |>
    dplyr::mutate(genus = trimws(genus, which = "both")) |>
    dplyr::mutate(species = gsub(
      pattern = "^\\S{1,}", replacement = "",
      x = .data$genus
    )) |>
    dplyr::mutate(species = trimws(species, which = "both")) |>
    dplyr::mutate(species = ifelse(isgenus & addsp,
      paste0(species, "sp."), species
    )) |>
    dplyr::mutate(tax = paste(genus, species)) |>
    dplyr::pull(.data[[type]])
  return(tidytaxname)
}
